/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.view.report.ReportViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("com.ibm.rational.report.web.core.app.internal.MessageFormatter");
dojo.require("com.ibm.rational.report.web.core.app.internal.Util");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.ErrorDialog");
dojo.require("com.ibm.rational.report.web.core.app.view.report.ReportParameterViewer");
dojo.require("com.ibm.rational.report.web.core.app.view.report.ReportParameterManager");
dojo.require("com.ibm.rational.report.web.core.app.view.report.ExportReportDialog");
dojo.require("com.ibm.rational.report.web.core.app.view.report.EmailReportDialog");
dojo.require("com.ibm.rational.report.web.core.app.view.report.SaveReportDialog");
dojo.require("com.ibm.rational.report.web.core.client.internal.ReportClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.view.report", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");

var CommonMessages	    	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
var ServiceabilityMessages	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
var ReportMessages			= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.view.report", "Messages");
var MessageFormatter		= com.ibm.rational.report.web.core.app.internal.MessageFormatter;
var Util					= com.ibm.rational.report.web.core.app.internal.Util;
var ReportClient			= com.ibm.rational.report.web.core.client.internal.ReportClient;
var session					= com.ibm.team.repository.web.client.session;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;

var SERVICE_URI				= "com.ibm.rational.report.common.IReportContentRestService/";

var SEND_ICON				= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/action_send.gif");
var PRINT_ICON				= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/action_print.gif");
var EXPORT_ICON				= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/action_export.gif");

var REPORT_DESIGN			= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_design.gif");
var REPORT_QUERY			= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_query.gif");
var REPORT_SNAPSHOT			= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_snapshot.gif");
var REPORT_DESIGN_SHARED	= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_design_shared.gif");
var REPORT_QUERY_SHARED		= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_query_shared.gif");
var REPORT_SNAPSHOT_SHARED	= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_snapshot_shared.gif");

/**
 * Displays the rendered reports in the main content area.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.view.report.ReportViewer",
	[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/view/report/templates/ReportViewer.html"),
	
	// Arguments:
	//     name - mandatory name of the report
	//     parameterName - optional array of parameter names
	//     parameterValue - optional array of parameter values
	//     collapseParameters - boolean
	constructor: function(args) {
		this._controller = args.controller;
		this._reportItemId = args.reportItemId;
		this._args = args;
	},
	
	postCreate: function() {
		this._msgFormatter = new MessageFormatter(this.messageArea);		
		var self = this;		
		var args = this._args;
		this.hasContent = false;
		this.printing = false;
		
		this.pageButtonContainer.style.visibility = 'hidden';
		this.saveAsContainer.style.visibility = 'hidden';
		
		var toolbar = new dijit.Toolbar({layoutAlign: "top", style: "background : transparent; border: none"});	
        
		if (args.type != "REPORT_DESIGN") {
			this.sendButton = new dijit.form.Button({
            	title: ReportMessages.REPORT_VIEWER_SEND_BUTTON_TOOLTIP,
            	iconClass: "EmailReportButtonIcon",
				label: ReportMessages.REPORT_VIEWER_SEND_BUTTON_TOOLTIP,
				showLabel: false,
            	onClick: dojo.hitch(this, this._emailReport)
        	});
			toolbar.addChild(this.sendButton);
		}
		
		this.printButton = new dijit.form.Button({
            title: ReportMessages.REPORT_VIEWER_PRINT_BUTTON_TOOLTIP,
            iconClass: "PrintReportButtonIcon",
			label: ReportMessages.REPORT_VIEWER_PRINT_BUTTON_TOOLTIP,
			showLabel: false,
            onClick: dojo.hitch(this, this._handlePrint)
        });
		toolbar.addChild(this.printButton);
		
		this.exportButton = new dijit.form.Button({
            title: ReportMessages.REPORT_VIEWER_EXPORT_BUTTON_TOOLTIP,
            iconClass: "ExportReportButtonIcon",
			label: ReportMessages.REPORT_VIEWER_EXPORT_BUTTON_TOOLTIP,
			showLabel: false,
            onClick: dojo.hitch(this, this._exportReport)
        });
		toolbar.addChild(this.exportButton);
		
		this.pageButtonContainer.appendChild(toolbar.domNode);
		
		if (args.type != "REPORT_SNAPSHOT") {
			this.saveAsButton = document.createElement("button");
			this.saveAsButton.appendChild(document.createTextNode(ReportMessages.REPORT_VIEWER_SAVE_AS_BUTTON_LABEL));
			this.saveAsButton.title = ReportMessages.REPORT_VIEWER_SAVE_AS_BUTTON_TOOLTIP;
			this.saveAsContainer.appendChild(this.saveAsButton);
			dojo.connect(this.saveAsButton, "onclick",
				function(e) {
					e.preventDefault();
					self._saveReport();
				}
			);
		}
		
		this._disableButtons(true);
		this._teamAreas = [];		
		this._projectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
		this._parseTeamAreas(this._projectArea.children);
		
		var requestParams = {
			id: args.reportItemId
		};
		
		Util.removeChildren(this.pageTitle);
		this._msgFormatter.displayLoadingMessage(); 
		
		var srh = new ServiceResponseHandler(this, "_displayTitle",	"_remoteError");
		if(args.type == "REPORT_DESIGN"){
			ReportClient.getReportDesign(srh, requestParams);
		} else if(args.type == "REPORT_QUERY"){
			ReportClient.getReportQuery(srh, requestParams);
		} else if(args.type == "REPORT_SNAPSHOT"){
			ReportClient.getReportSnapshot(srh, requestParams);
		}
	},
		
	_disableButtons: function(disabled) {
		if (this.sendButton != null)
			this.sendButton.setDisabled(disabled);
		this.printButton.setDisabled(disabled);
		this.exportButton.setDisabled(disabled);
		if (this.saveAsButton != null )
			this.saveAsButton.disabled = disabled;
	},
	
	_displayTitle: function(report) {
		this._report = report;		
		var self = this;
		var args = this._args;		
		
		var pageTitleIcon = document.createElement("img");
		this.pageTitle.appendChild(pageTitleIcon);
		if(args.type == "REPORT_DESIGN")
			pageTitleIcon.src = this._report.shared ? REPORT_DESIGN_SHARED : REPORT_DESIGN;
		else if(args.type == "REPORT_QUERY")
			pageTitleIcon.src = this._report.shared ? REPORT_QUERY_SHARED : REPORT_QUERY;
		else if(args.type == "REPORT_SNAPSHOT")
			pageTitleIcon.src = this._report.shared ? REPORT_SNAPSHOT_SHARED : REPORT_SNAPSHOT;
		this.pageTitle.appendChild(document.createTextNode(this._report.name.replace(" ", "\xa0", "g"))); // preserve the spaces
		
		var args = {id: this._args.reportItemId};
		if (this._args.design != null) {
			args.design = this._args.design;
			this._design = args.design;
		}
		
		if(report.reportDesignType != 'CRYSTAL_REPORTS'){
			this.pageButtonContainer.style.visibility = 'visible';
			this.saveAsContainer.style.visibility = 'visible';
		}
		
		var srh = new ServiceResponseHandler(this, "_displayReportParameters",	"_remoteError");
		ReportClient.getReportParameterDefs(srh, args);
	},
	
	_displayReportParameters: function(parameterDefs){
		this._msgFormatter.displayNoMessage();
		var design = null;
		if (this._args.design != null) {
			design = this._args.design;
		}
		this._parameterManager = new com.ibm.rational.report.web.core.app.view.report.ReportParameterManager(
			this._report, design, parameterDefs, this._args.reportParamNames, this._args.reportParamValues);
		if(this._parameterManager.hasVisibleParams()){
			this._parameterViewer = new com.ibm.rational.report.web.core.app.view.report.ReportParameterViewer({
				paramManager: this._parameterManager});
			this.parametersDiv.appendChild(this._parameterViewer.domNode);
		}
		if(this._parameterManager.hasAllRequiredValues()){
			this._getRenderedReport();
		}
		var self = this;
		dojo.subscribe("com.ibm.rational.report.runReport."+this._parameterManager.report.itemId, 
			function() {
				self._getRenderedReport();
			}
		);
	},
	
	_getRenderedReport: function() {
		if (!this._parameterManager.hasAllRequiredValues()) {
			this._displayErrorMessage(ServiceabilityMessages.ALL_REPORT_PARAMETERS_ARE_REQUIRED);
   			return;
		}
		this._msgFormatter.displayLoadingMessage(); 
		var args = {id: this._args.reportItemId};
		if (this._args.design != null) {
			args.design = this._args.design;
			this._design = args.design;
		}
		var parameters = this._parameterManager.getParamNameAndValues();
		args.reportParamNames = parameters.names;
		args.reportParamValues = parameters.values;
		var srh = new ServiceResponseHandler(this, "_displayRenderedReport",	"_remoteError");
		ReportClient.getRenderedReport(srh, args);
	},
	
	_parseTeamAreas: function(teamAreas) {	
		if (teamAreas == null || teamAreas.length == 0) {
			return;
		}
		for (var i = 0; i < teamAreas.length; i++) {
			var teamArea = teamAreas[i];
			this._teamAreas.push(teamArea.name);		
			this._parseTeamAreas(teamArea.children);
		}
	},
	
	_handlePrint: function() {
		if(this.hasContent){
			this._printReport();
		} else {
			this.printing = true;
			this._getRenderedReport();
		}
	},
	
	_printReport: function() {
		this.printing = false;
		var name = "print";
		var availHeight = screen.availHeight;
		var availWidth  = screen.availWidth+1000;
		var w = window.open(null, name, 'toolbar=no,resizable=yes,scrollbars=1,left=0,top=0,height='+availHeight+'width='+availWidth);
		var doc = w.document;
		doc.open();
		var innerHtml = "<head>";
		innerHtml += "<title>"+this._report.name+"</title>";
		innerHtml += "<style type='text/css' media='screen'>.commandbar {width: '100%'; border-bottom: #999999 solid 1px}</style>";
		innerHtml += "<style type='text/css' media='print'>.commandbar {display: none}</style>";
		innerHtml += "</head>";
		innerHtml += "<body>";
		innerHtml += "<table class='commandbar' width='100%'><tr>";
		innerHtml += "<td align='left'><a href='javascript: window.print()'>"+ReportMessages.PRINT_PREVIEW_PRINT_BUTTON+"</a></td>";
		innerHtml += "<td align='right'><a href='javascript: window.close()'>"+ReportMessages.PRINT_PREVIEW_CLOSE_BUTTON+"</a></td>";
		innerHtml += "</tr></table>";
		innerHtml += "<h1>"+this._report.name+"</h1>";
		innerHtml += this.renderedReportDiv.innerHTML;
		innerHtml += "</body>";
		doc.write(innerHtml);
		doc.close();
	},
	
	_saveReport: function() {
		if (!this._parameterManager.hasAllRequiredValues()) {
			this._displayErrorMessage(ServiceabilityMessages.ALL_REPORT_PARAMETERS_ARE_REQUIRED);
   			return;
		}
		
		var self = this;
		var saveReportDialog = new com.ibm.rational.report.web.core.app.view.report.SaveReportDialog(
			{controller: this._controller, title: ReportMessages.SAVE_REPORT_VIEWER_TITLE});
		dojo.connect(saveReportDialog.saveButton, "onclick", saveReportDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				self._saveReportName = dojo.trim(saveReportDialog.nameText.value);
				var name = self._saveReportName;
				for(var i=0;i<name.length;i++){
					if(name.charCodeAt(i) == 47 || name.charCodeAt(i) == 92 || name.charCodeAt(i) == 42 || name.charCodeAt(i) == 63 ||
			   			name.charCodeAt(i) == 124 || name.charCodeAt(i) == 62 || name.charCodeAt(i) == 60 || name.charCodeAt(i) == 58  || name.charCodeAt(i) == 34){
						self._displayErrorMessage(CommonMessages.ERROR_FILE_NAME+ ": * \\\ / | \  : < \" > ? ");
						return;
					}
				}
				self._handleSave(saveReportDialog.typeCombo.value, saveReportDialog.nameText.value,
					saveReportDialog.descriptionText.value, saveReportDialog.parentFolder, saveReportDialog.sharedCheckbox.checked);
				saveReportDialog.close();
			}
		);	
		dojo.connect(saveReportDialog.closeButton, "onclick", saveReportDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				saveReportDialog.close();
			}
		);
	},
	
	_handleSave: function(type, name, description, parentFolder, shared) {
		var self = this;
		
		this._msgFormatter.displayLoadingMessage();
		this._disableButtons(true);

		this._parentFolderId = "";
		
		if (parentFolder.selectedIndex >= 0) {
			this._parentFolderId = parentFolder.options[parentFolder.selectedIndex].value;
		}

		var owner = session.getAuthenticatedContributor();
		var projectAreaId = this._projectArea.itemId;		
		if (typeof teamAreaId == "undefined" || teamAreaId == null) {
			teamAreaId = "";
		}
		
		var id = this._report.itemId;
		if (typeof this._report.reportDesign != "undefined" && this._report.reportDesign != null) {
			id = this._report.reportDesign.itemId;
		}

		var args = {
			name: name,
			description: description,
			reportDesignId: id,
			projectAreaId: projectAreaId,
			teamAreaId: teamAreaId,
			shared: shared,
			ownerId: owner.itemId
		};
		
		var parameters = this._parameterManager.getParamNameAndValues();
		args.reportParamNames = parameters.names;
		args.reportParamValues = parameters.values;
		
		if (type == "REPORT_QUERY") {
			var srh = new ServiceResponseHandler(this, "_createReportTreeNode", "_createReportQueryError");			
			ReportClient.postNewReportQuery(srh, args);
		}
		else if (type == "REPORT_SNAPSHOT") {
			var srh = new ServiceResponseHandler(this, "_createReportTreeNode", "_createReportSnapshotError");				
			ReportClient.postNewReportSnapshot(srh, args);
		}
	},
	
	_createReportTreeNode: function(result) {			
		var self = this;
		
		var postExec = function() {			
			self._controller.syncReportNodeChanged();			
		};
		
		this._postServiceExec = postExec;
		
		var srh = new ServiceResponseHandler(this, "_done",	"_remoteError");
		
		var projectAreaId = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea().itemId;
	    var ownerId = session.getAuthenticatedContributor().itemId;

		var params = { reportItemId: result.itemId, parentFolderId: this._parentFolderId, projectAreaId: projectAreaId, ownerId: ownerId };						
		ReportClient.postNewReportTreeNode(srh, params);
	},
	
	_createReportQueryError: function(error) {
		this._msgFormatter.displayNoMessage();
		this._disableButtons(false);	
		var msg = dojo.string.substitute(ServiceabilityMessages.FAILED_TO_CREATE_REPORT_QUERY, [this._saveReportName]);
		var errMsg = this._msgFormatter.formatErrorMessage(msg, error); 
		this._displayErrorMessage(errMsg);
	},
	
	_createReportSnapshotError: function(error) {
		this._msgFormatter.displayNoMessage();
		this._disableButtons(false);
		var msg = dojo.string.substitute(ServiceabilityMessages.FAILED_TO_CREATE_REPORT_SNAPSHOT, [this._saveReportName]);
		var errMsg = this._msgFormatter.formatErrorMessage(msg, error);
		this._displayErrorMessage(errMsg);
	},		
	
	_emailReport: function() {
		var self = this;
		var emailReportDialog = new com.ibm.rational.report.web.core.app.view.report.EmailReportDialog(
			{title: ReportMessages.EMAIL_REPORT_VIEWER_TITLE});
		dojo.connect(emailReportDialog.sendButton, "onclick", emailReportDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				self._handleSend(emailReportDialog.to.value, emailReportDialog.cc.value,
					emailReportDialog.subject.value, emailReportDialog.message.value);
				emailReportDialog.close();
			}
		);	
		dojo.connect(emailReportDialog.closeButton, "onclick", emailReportDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				emailReportDialog.close();
			}
		);
	},
	
	_handleSend: function(to, cc, subject, message) {
		var self = this;
		
		this._msgFormatter.displayLoadingMessage();
		this._disableButtons(true);

		var args = {
			reportItemId: this._reportItemId,
			to: to,
			cc: cc,
			subject: subject,
			message: message
		};
		
		var srh = new ServiceResponseHandler(this, "_done", "_sendError");
		ReportClient.postEmailReport(srh, args);
	},
		
	_done: function(result) {
		this._msgFormatter.displayNoMessage();
		this._disableButtons(false);
		this._controller.reportTreeParser.expandFolder(this._parentFolderId);
		if (this._postServiceExec != null) {
			this._postServiceExec();
		}
	},
	
	_exportReport: function() {
		if (this._args.type != "REPORT_SNAPSHOT" && !this._parameterManager.hasAllRequiredValues()) {
			this._displayErrorMessage(ServiceabilityMessages.ALL_REPORT_PARAMETERS_ARE_REQUIRED);
   			return;
		}
		
		var self = this;
		var exportReportDialog = new com.ibm.rational.report.web.core.app.view.report.ExportReportDialog(
			{title: ReportMessages.EXPORT_REPORT_VIEWER_TITLE});
		dojo.connect(exportReportDialog.exportButton, "onclick", exportReportDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				self._handleExport(exportReportDialog.formatCombo.value);
				exportReportDialog.close();
			}
		);	
		dojo.connect(exportReportDialog.closeButton, "onclick", exportReportDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				exportReportDialog.close();
			}
		);
	},
	
	_handleExport: function(format) {
		var args = {
			id: this._reportItemId,
			format: format
		};
		
		if (this._args.type != "REPORT_SNAPSHOT") {
			var parameters = this._parameterManager.getParamNameAndValues();
			args.reportParamNames = parameters.names;
			args.reportParamValues = parameters.values;
			if (this._design != null) args.design = this._design;
		}
		
		this._msgFormatter.displayLoadingMessage();
		this._disableButtons(true);
		var srh = new ServiceResponseHandler(this, "_downloadExportedReport", "_remoteError");
		ReportClient.postExportReport(srh, args);
	},
	
	_downloadExportedReport: function(result) {
		this._msgFormatter.displayNoMessage();
		this._disableButtons(false);
		location.href = TeamServerClient._getServicePrefix() + SERVICE_URI + "outputs/" + result;
	},

	_displayRenderedReport: function(result) {
		this._msgFormatter.displayNoMessage();	
		this._disableButtons(false);	
		if (navigator.appVersion.indexOf("MSIE")!=-1) {
			// Insert innerHTML directly. That, coupled with defer tags
			// on the scripts, causes new Javascript to be executed properly.
			this.renderedReportDiv.innerHTML = result;
		} else {
			// Create a new node and appendChild so javascript gets executed.
			var newNode = document.createElement("div");
			newNode.innerHTML = result;
			Util.removeChildren(this.renderedReportDiv);
			this.renderedReportDiv.appendChild(newNode);
		} 
		this.hasContent = true;
		if(this.printing) this._printReport();
	},
	
	_postActionFunc: function(){
		this._controller.applySelectedTeamAreas();
	},

	_remoteError: function(error) {
		this._disableButtons(false);
		this._msgFormatter.displayNoMessage();
		var msg = this._msgFormatter.formatErrorMessage(ServiceabilityMessages.FAILED_TO_RENDER_REPORT, error);
		this._displayErrorMessage(msg);
		this._controller.clearMainContent();
	},
	
	_sendError: function(error) {
		this._disableButtons(false);
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_SEND_REPORT_EMAIL, error);
	},
	
	_isArray: function(obj) {
	   if (obj.constructor.toString().indexOf("Array") == -1)
	      return false;
	   else
	      return true;
	},
	
	_displayErrorMessage: function(errorMsg) {
		var errorDialog = new com.ibm.rational.report.web.core.app.internal.common.ErrorDialog();
		errorDialog.displayError(errorMsg);
	}
	
});

})();